#ifndef MRK_LEXER
#define MRK_LEXER

#include <stdbool.h>

#include "mrk/common.h"

typedef struct mrk_lexer mrk_lexer;

typedef enum mrk_lexer_err {
  mrk_lexer_err_ok = 0,
  mrk_lexer_err_done,
  mrk_lexer_err_unexpected_char,
} mrk_lexer_err;

typedef enum mrk_token_type {
  mrk_token_type_none = 0,
  mrk_token_type_pounds,
  mrk_token_type_backticks,
  mrk_token_type_dashes,
  mrk_token_type_underscores,
  mrk_token_type_stars,
  mrk_token_type_equals,
  mrk_token_type_blank_line,
  mrk_token_type_newline,
  mrk_token_type_spaces,
  mrk_token_type_line_break,
  mrk_token_type_right_angle_brackets,
  mrk_token_type_tabs,
  mrk_token_type_left_bracket,
  mrk_token_type_right_bracket,
  mrk_token_type_bang,
  mrk_token_type_left_paren,
  mrk_token_type_right_paren,
  mrk_token_type_backslash,
  mrk_token_type_dotted_number,
  mrk_token_type_word,
  mrk_token_type_checked_box,
  mrk_token_type_unchecked_box,
  mrk_token_type_text,
} mrk_token_type;

typedef struct mrk_token {
  mrk_token_type type;
  size_t start;
  size_t end;
} mrk_token;

/**
 * Return the length of the given token.
 */
size_t mrk_token_len(mrk_token t);

/**
 * Return a pointer to the start of the lexeme in the initial buffer, returning
 * the length of the lexeme.
 */
size_t mrk_token_lexeme(const char **out, mrk_lexer *lexer, mrk_token t);

/**
 * Initialize a new lexer struct.
 */
mrk_err mrk_lexer_init(mrk_lexer **out);

/**
 * Deallocate the lexer object.
 */
void mrk_lexer_free(mrk_lexer *lexer);

/**
 * Open the buffer with the given lexer struct. `buf` is expected to live for
 * the duration of the lexing.
 *
 * The lexer will run either until `len` characters have been matched, or until
 * a nul character has been reached. If `len` is set to 0, only the nul check is
 * used to determine the end of the buffer.
 */
void mrk_lexer_open(mrk_lexer *lexer, const char *buf, size_t len);

/**
 * Returns whether the lexer is done.
 */
bool mrk_lexer_done(const mrk_lexer *lexer);

/**
 * Output the next lexed token for the given input.
 */
mrk_lexer_err mrk_lexer_next(mrk_token *out, mrk_lexer *lexer);

#endif
